#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

#include "mysqlstuff.h" /* Currently assumes a MYSQL database */

#define DEBUG /* MACRO to make debugging easier */
#define DEBUGLEVEL 3 /* only for ldebug calls */

#define GEQUELVERSION " v0.91 BETA "

typedef struct _Params
{
   char      *table;      /* currently selected table */
   int       numflds;     /* number of fields (without opt. indices) */
   int       sortkey;     /* number of field we'll use to sort */

   SQL       *dbsock;     /* MYSQL style : int dbsock for MSQL */
   SQL_FIELD *fields;     /* stores fields for current table */
   
   GtkText   *logwindow;  /* widget main window: need it for log messages */
   GtkStyle  *defstyle;
   GtkStyle  *sortstyle;
   GtkStyle  *boldstyle;
   GtkStyle  *boldsortstyle;

   /* until here we overlap with the Record struct : consider using union ? */
   
   char      *user;
   char      *pwd;
   char      *server;
   char      *database;
   int       numrecs;
   int       countflag;
   char      browserinuse; /* flag to know if browser is showing records */
   int       selectedrow;  /* number of selected row (or -1 if none) */
      
   GtkWidget *mainwindow;
   GtkText   *querywindow;
   GtkWidget *browserwindow;
   GtkWidget *browserscroller;
   GtkWidget *browserframe;

   GtkOptionMenu *tablelist;
   GtkOptionMenu *dbaselist;
} Params;

typedef struct _Record
{
   char      *table;      /* currently selected table */
   int       numflds;     /* number of fields (without opt. indices) */
   int       sortkey;     /* number of field we'll use to sort */

   SQL       *dbsock;     /* MYSQL style : int dbsock for MSQL */
   SQL_FIELD *fields;     /* stores fields for current table */

   GtkText   *logwindow;  /* widget main window: need it for log messages */
   GtkStyle  *defstyle;
   GtkStyle  *sortstyle;
   GtkStyle  *boldstyle;
   GtkStyle  *boldsortstyle;

   /* until here we overlap with the Params struct : consider using union ? */

   char      **contents;  /* field contents as obtained from database */
   GtkWidget *window;     /* top window for this record */
   GtkWidget *gtable;     /* holds info: do not confuse with dbase table ! */
   GtkEntry  **entries;   /* (changed) field contents inside GUI textentry */
} Record;

#define SELECT "SELECT * FROM %s "
#define INSERT "INSERT INTO %s VALUES ("
#define UPDATE "UPDATE %s SET "
#define DELETE "DELETE FROM %s "
#define WHERE  " WHERE "
#define EQUAL  "%s = %s%s%s"
#define LIKE   "%s LIKE %s%s%s"
#define AND    " AND "
#define ORDER  "ORDER BY %s "

#define PASSIVE  0
#define ACTIVE   1
#define KEYFIELD 2

#define COUNT    0
#define NOCOUNT  1

#define MAXLOGSIZE 50000
#define MAXQUERY   10000
#define MAXMESSAGE 1024

/* main.c : */
void refreshBrowser();

/* startup.c */
void gequelInit(Params *p, int argc, char *argv[]);
void storeWidgets(GtkWidget *main, Params *p);
void connectMainWindowSignals(GtkWidget *main, Params *p);

/* dialog.c : */
void debug(char *format, ...);
void ldebug(int level, char *format, ...);
void defaultAction(GtkButton *button, gpointer dummy);
void createMessageBox(char *s);
void createConfirmBox(char *s, gpointer yes, gpointer no, gpointer hook);
void showMessage(char *s,...);
void showSqlError(GtkText *logwindow, SQL *sock, char *s,...);
void writeLog(GtkText *logwindow, char *s,...);

/* helpers.c : */
int getIntValue(GtkWidget *widget, char *widgetname);
char *getStringValue(GtkWidget *widget, char *widgetname);
void setStringValue(GtkWidget *widget, char *widgetname,char *string);
int executeQuery(GtkText *logwindow, SQL *sock, char *query);
int getNumRecords(Params *p);
void setGUICounters(Params *p, int nrFields, int nrRecords);
int getSortKey(Params *p);
void closeTopWindow(GtkWidget *child);
GtkStyle *createFontStyle(GtkStyle *source, gchar *fontname);
GtkStyle *createColorStyle(GtkStyle *source, double rgb[3]);
GtkStyle *createColorFontStyle(GtkStyle *source, double rgb[3], gchar *fontname);

/* sqlconnect.c : */
void setNewServer(Params *p, char *server);
void setDBaseList(Params *p, char *preselect);
void setTableList(Params *p, char *preselect);
void setNewDatabase(Params *p, char *database);
void setNewTable(Params *p, char *table);

/* showrecords.c : */
void destroyBrowser(Params *p);
void deleteRecord(Params *p, int row);
void createBrowser(Params *p, gchar **titles, void *callback, int numflds);

void showRecords(Params *p);

/* showfields.c : */
void showFields(Params *p);

/* recordform.c : */
void on_recorddelete_clicked(GtkButton *dummy, gpointer rec); /* callback */
Record *createRecordstruct(Params *p);
void destroyRecordWindow(Record *rec);
void createModifyWindow(Params *p);
void createCopyRecordWindow(Record *rec);
void createCopyMainWindow(Params *p);
void createInsertWindow(Params *p);
void createSearchWindow(Params *p);

/* sqlactions.c : */
void updateRecordAction(GtkButton *button, gpointer rec);
void deleteRecordAction(GtkButton *button, gpointer rec);
void insertRecordAction(GtkButton *button, gpointer rec);

