#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

/* First the callbacks for RecordWindow : */

/* Record menus : */
void on_record_menusave_activate(GtkMenuItem *menuitem, gpointer rec)
{
   ;
}

void on_record_menuclose_activate(GtkMenuItem *menuitem, gpointer rec)
{
   destroyRecordWindow((Record *)rec);
}

/* Record buttons : */
void on_recordclose_clicked(GtkButton *button, gpointer rec)
{
   destroyRecordWindow((Record *)rec);
}

void on_recordupdate_clicked(GtkButton *button, gpointer rec)
{
   ldebug(2,"on_recordupdate_clicked entered\n");
   
   createConfirmBox("Really update selected record ?\n",
		    GTK_SIGNAL_FUNC (updateRecordAction), NULL, rec);
}

void on_recordcopy_clicked(GtkButton *button, gpointer rec)
{
   createCopyRecordWindow((Record *)rec);
}

void on_recorddelete_clicked(GtkButton *button, gpointer rec)
{
   ldebug(2,"on_recorddelete_clicked entered\n");
   
   createConfirmBox("Really delete selected record ?\n",
		    GTK_SIGNAL_FUNC (deleteRecordAction), NULL, rec);
}

void on_recordrestore_clicked(GtkButton *button, gpointer rec)
{
   int y;
   Record *r = (Record *)rec;
   /* Restore the original field contents of the selected record : */
   for(y=0;y<r->numflds;y++)
      gtk_entry_set_text(r->entries[y], r->contents[y]);
}

void on_recordinsert_clicked(GtkButton *button, gpointer rec)
{
   ldebug(2,"on_recordinsert_clicked entered\n");
   
   createConfirmBox("Really insert record ?\n",
		    GTK_SIGNAL_FUNC (insertRecordAction), NULL, rec);
}

void on_recordsearch_clicked(GtkButton *button, gpointer rec)
{
   ;
}

void on_recordprev_clicked(GtkButton *button, gpointer rec)
{
   ;
}

void on_recordnext_clicked(GtkButton  *button, gpointer rec)
{
   ;
}

/* Next ensures that every button or menu action has a pointer to the 
   record structure in "user_data". In the glade generated interface.c it
   is impossible to hook the record without modifying create_recordwindow
   which makes updates using glade very complicated !
*/
void connectRecordWindowSignals(Record *rec)
{
   ldebug(2,"connectRecordWindowSignals entered\n");
   ldebug(3,"Record pointer: %ld\n",(long)rec);
   
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"record_menusave")),
		       "activate",GTK_SIGNAL_FUNC (on_record_menusave_activate),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"record_menuclose")),
		       "activate",GTK_SIGNAL_FUNC (on_record_menuclose_activate),
		       (gpointer)rec);
   
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordclose")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordclose_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordupdate")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordupdate_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordcopy")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordcopy_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recorddelete")),
		       "clicked",GTK_SIGNAL_FUNC (on_recorddelete_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordrestore")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordrestore_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordinsert")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordinsert_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordsearch")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordsearch_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordprev")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordprev_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordnext")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordnext_clicked),
		       (gpointer)rec);
}

/* Stores all information in a record, so the recordwindow won't fail or crash
 * in case the main window has switched to a different table. Different
 * database is too complicated : in that case we should even consider to 
 * destroy all open recordwindows. (but what if table is changed back ?
 * database will issue errors or warnings anyway !) */
Record *createRecordstruct(Params *p)
{
   int y;
   int tlen;
   Record *rec;
   gchar *text;
   
   ldebug(2,"createRecordstruct entered\n");
   
   if(p->table == NULL)
     return NULL;
   rec = (Record *)g_malloc(sizeof(Record));
   if(rec == NULL)
     return NULL;

   rec->numflds    = p->numflds;
   rec->dbsock     = p->dbsock;
   rec->sortkey    = p->sortkey;
   rec->logwindow  = p->logwindow;

   rec->defstyle   = p->defstyle;
   rec->sortstyle  = p->sortstyle;
   rec->boldstyle  = p->boldstyle;
   rec->boldsortstyle  = p->boldsortstyle;

   tlen = strlen(p->table) + 1;
   rec->table = (char *)g_malloc(tlen);
   strncpy(rec->table,p->table,tlen);

   /* create field and content buffers for later use : */
   rec->fields   = (SQL_FIELD *)g_malloc(sizeof(SQL_FIELD) * p->numflds);
   rec->contents = (char **)g_malloc(sizeof(char *) * p->numflds);
   if(rec->contents == NULL || rec->fields == NULL)
   {
      showMessage("Memory error in create Record (buffers) for %s !",
		  p->table);
      exit(1);
   }
   
   /* Copy complete field array from Params : */
   memcpy(rec->fields,p->fields,sizeof(SQL_FIELD) * p->numflds);

   for(y=0;y<p->numflds;y++)
   {
      /* Copy pointer "name" as well. Note that we do not copy other strings 
       like "table" and "def" here, so fieldinfo is incomplete ! */
      
      tlen = strlen(p->fields[y].name) + 1;
      rec->fields[y].name   = (char *)g_malloc(tlen);
      if(rec->fields[y].name == NULL)
      {
	 showMessage("Memory error in create Record (names) for %s !",
		     p->table);
	 exit(1);
      }
      strncpy(rec->fields[y].name, p->fields[y].name, tlen);
      
      tlen = p->fields[y].length + 1;
      rec->contents[y] = (char *)g_malloc(tlen); /* check ! */
      /* Copy contents to record structure (for Modify, Copy & Restore): */
      if(p->selectedrow >= 0)
      {
	 gtk_clist_get_text(GTK_CLIST(p->browserwindow),
			    p->selectedrow, y, &text);
	 strncpy(rec->contents[y],text,tlen);
      }
   }
   
   /* do we still need this ? yes : for reading the changed fieldcontents */
   rec->entries = (GtkEntry **)g_malloc(sizeof(GtkEntry *) * p->numflds);

   rec->window = NULL; /* NULL until topwindow created */
   return rec;
}

Record *copyRecordstruct(Record *src)
{
   int y;
   int tlen;
   Record *dst;
   
   ldebug(2,"copyRecordstruct entered\n");
   
   if(src == NULL)
     return NULL;
   dst = (Record *)g_malloc(sizeof(Record));
   if(dst == NULL)
     return NULL;

   dst->numflds    = src->numflds;
   dst->dbsock     = src->dbsock;
   dst->sortkey    = src->sortkey;
   dst->logwindow  = src->logwindow;

   dst->defstyle   = src->defstyle;
   dst->sortstyle  = src->sortstyle;
   dst->boldstyle  = src->boldstyle;
   dst->boldsortstyle  = src->boldsortstyle;

   tlen = strlen(src->table) + 1;
   dst->table = (char *)g_malloc(tlen);
   strncpy(dst->table,src->table,tlen);

   /* copy fieldnames and create contentbuffer for later use : */
   dst->fields   = (SQL_FIELD *)g_malloc(sizeof(SQL_FIELD) * src->numflds);
   dst->contents = (char **)g_malloc(sizeof(char *) * src->numflds);
   if(dst->contents == NULL || dst->fields == NULL)
   {
      showMessage("Memory error in copy Record (buffers) for %s !",
		  src->table);
      exit(1);
   }
   /* Copy complete field array from src : */
   memcpy(dst->fields,src->fields,sizeof(SQL_FIELD) * src->numflds);

   for(y=0;y<src->numflds;y++)
   {
      /* Copy pointer "name" as well. Note that we do not copy other strings 
       like "table" and "def" here, so fieldinfo is incomplete ! */
      
      tlen = strlen(src->fields[y].name) + 1;
      dst->fields[y].name   = (char *)g_malloc(tlen);
      if(dst->fields[y].name == NULL)
      {
	 showMessage("Memory error in copy Record (names) for %s !",
		     src->table);
	 exit(1);
      }
      strncpy(dst->fields[y].name, src->fields[y].name, tlen);

      tlen = src->fields[y].length + 1;
      dst->contents[y] = (char *)g_malloc(tlen);
      /* don't copy from entries but from contents so we'll get changed 
       * fields in the source as well : */
      strncpy(dst->contents[y],gtk_entry_get_text(src->entries[y]),tlen);
   }
   
   /* do we still need this ? yes : for reading the changed fieldcontents */
   dst->entries = (GtkEntry **)g_malloc(sizeof(GtkEntry *) * src->numflds);

   return dst;
}

void destroyRecordstruct(Record *rec)
{
   int y;
   
   ldebug(2,"destroyRecordstruct entered\n");
   
   for(y=0; y<rec->numflds; y++)
   {
      g_free(rec->fields[y].name);
      g_free(rec->contents[y]);
   }
   g_free(rec->fields);
   g_free(rec->contents);
   g_free(rec->table);
   g_free(rec->entries);
   
   g_free(rec);
}

void destroyRecordWindow(Record *rec)
{
   ldebug(2,"destroyRecordwindow entered\n");
   ldebug(3,"Record pointer: %ld\n",(long)rec);

   if(rec->window != NULL) /* delete record doesn't need to have a window ! */
   {
      gtk_widget_hide(rec->window);
      gtk_widget_destroy(rec->window);
   }
   
   destroyRecordstruct(rec);
}

/* Create an empty recordwindow */
void createEmptyWindow(Record *rec)
{
   int y;
   GtkWidget *frame;
   GtkWidget *label;
   GtkWidget *entry;
   
   ldebug(2,"createEmptyWindow entered\n");
   
   rec->window = create_recordwindow();
   frame       = GTK_WIDGET(lookup_widget (rec->window,"recordframe"));
   rec->gtable = gtk_table_new(rec->numflds, 2, TRUE); /* 2 columns, n rows */

   for(y=0;y<rec->numflds;y++)
   {
      label = gtk_label_new (rec->fields[y].name);
      gtk_widget_ref (label);
      if(y == rec->sortkey)
        gtk_widget_set_style(label, rec->boldsortstyle);
      else
	gtk_widget_set_style(label, rec->boldstyle);
      gtk_widget_show (label);
      gtk_table_attach_defaults (GTK_TABLE (rec->gtable), label, 0,1, y, y+1);
      
      entry = gtk_entry_new ();
      gtk_widget_ref (entry);
      gtk_widget_show (entry);
      gtk_table_attach_defaults (GTK_TABLE (rec->gtable), entry, 1,2,  y, y+1);
      
      rec->entries[y] = GTK_ENTRY(entry);
   }
   
   gtk_table_set_row_spacings (GTK_TABLE (rec->gtable), 10);
   gtk_widget_show(rec->gtable);
   gtk_container_set_border_width (GTK_CONTAINER (frame), 20);
   gtk_container_add (GTK_CONTAINER (frame), rec->gtable);

   connectRecordWindowSignals(rec);
   gtk_widget_hide(rec->window);  /* turn on later when all is finished */
}

void createInsertWindow(Params *p)
{
   Record *rec;
   
   ldebug(2,"createInsertWindow entered\n");

   if(p->table == NULL)
   {
      showMessage("Please select a table first !\n");
      return;
   }
   
   rec = createRecordstruct(p);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Insert Record window !!\n");
      return;
   }
   
   createEmptyWindow(rec);
   
   /* Turn on the buttons we need here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordinsert")));
   /* gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordclear"))); */
   gtk_widget_show(rec->window);
}

void createModifyWindow(Params *p)
{
   int y;
   Record *rec;
   
   ldebug(2,"createModifyWindow entered\n");
   
   if(p->browserinuse == FALSE || p->selectedrow < 0)
   {
      showMessage("Please open browser and/or select a record to modify !\n");
      return;
   }
   
   rec = createRecordstruct(p);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Modify Record window !!\n");
      return;
   }
   
   createEmptyWindow(rec);
   
   /* Turn on the buttons we want here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recorddelete")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordupdate")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordcopy")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordrestore")));
   
   /* Copy the field contents of the selected record to entries : */
   for(y=0;y<rec->numflds;y++)
   {
      ldebug(2,"field %2d) %s: %s\n",y,rec->fields[y].name,rec->contents[y]);
      gtk_entry_set_text(rec->entries[y], rec->contents[y]);
   }
   gtk_widget_show(rec->window);
}

/* If clicked on Copy from record window : */
void createCopyRecordWindow(Record *src)
{
   int y;
   Record *rec;
   
   ldebug(2,"createCopyRecordWindow entered\n");
   
   rec = copyRecordstruct(src);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Copy Record window !!\n");
      return;
   }
   createEmptyWindow(rec);
   
   /* Turn on the buttons we need here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordinsert")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordrestore")));
   /* gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordclear")));*/
   
   /* Copy the field entry contents of the source record to destination
    * entries to ensure we always have the latest information : */
   for(y=0;y<rec->numflds;y++)
   {
      ldebug(2,"field %2d) %s: %s\n",y, rec->fields[y].name, rec->contents[y]);
      gtk_entry_set_text(rec->entries[y], rec->contents[y]);
   }
   gtk_widget_show(rec->window);
}

/* If clicked on Copy from mainwindow : */
void createCopyMainWindow(Params *p)
{
   int y;
   Record *rec;
   
   ldebug(2,"createCopyMainWindow entered\n");

   if(p->browserinuse == FALSE || p->selectedrow < 0)
   {
      showMessage("Please open browser and/or select a record to copy !\n");
      return;
   }
   
   rec = createRecordstruct(p);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Copy Main window !!\n");
      return;
   }
   createEmptyWindow(rec);
   
   /* Turn on the buttons we want here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordinsert")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordrestore")));
   /* gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordclear")));*/
   
   /* Copy the field contents of the selected record to entries : */
   for(y=0;y<rec->numflds;y++)
   {
      ldebug(2,"field %2d) %s: %s\n",y, rec->fields[y].name, rec->contents[y]);
      gtk_entry_set_text(rec->entries[y], rec->contents[y]);
   }
   gtk_widget_show(rec->window);
}

void createSearchWindow(Params *p)
{
   Record *rec;
   
   ldebug(2,"createSearchWindow entered\n");

   if(p->table == NULL)
   {
      showMessage("Please select a table first !\n");
      return;
   }
   
   rec = createRecordstruct(p);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Search Record window !!\n");
      return;
   }
   
   createEmptyWindow(rec);
   
   /* Turn on the buttons we need here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordsearch")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordprev")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordnext")));
   gtk_widget_show(rec->window);
}

