#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

void setSortKey(Params *p, int row)
{
   p->sortkey = row;
   showFields(p);
}

/* If we come here, then the user has selected a row in the list. */
void field_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
   Params *p = (Params *)data;

   ldebug(2,"field_selected entered\n");

   /* This only catches a doubleclick !! : */
   if (event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"field_selected: doubleclick detected\n");
      
      if(p->table == NULL)
      {
	 showMessage("Currently no table selected !\n");
	 return;
      }
      setSortKey(p, row);
      writeLog(p->logwindow, "Sort field now set to field %d (%s)\n",
	       row,p->fields[row].name);
      return;
   }
   else
   {
      ldebug(3,"You selected field %d: no effect (use doubleclick !)\n",row);
   }
}

gchar *getFieldTypeString(Params *p, int index)
{
   switch(p->fields[index].type)
   {
    case FIELD_TYPE_STRING:     return "String";    break;
    case FIELD_TYPE_VAR_STRING: return "VarString"; break;
    case FIELD_TYPE_FLOAT:      return "Float";     break;
    case FIELD_TYPE_DOUBLE:     return "Double";    break;
    case FIELD_TYPE_LONGLONG:
    case FIELD_TYPE_INT24:
    case FIELD_TYPE_DECIMAL:
    case FIELD_TYPE_TINY:
    case FIELD_TYPE_SHORT:
    case FIELD_TYPE_LONG:       return "Integer";   break;
    case FIELD_TYPE_DATE:       return "Date";      break;
    case FIELD_TYPE_TIME:       return "Time";      break;
    case FIELD_TYPE_TIMESTAMP:  return "TimeStamp"; break;
    case FIELD_TYPE_DATETIME:   return "Datetime";  break;
    case FIELD_TYPE_YEAR:       return "Year";      break;
    case FIELD_TYPE_TINY_BLOB:
    case FIELD_TYPE_MEDIUM_BLOB:
    case FIELD_TYPE_LONG_BLOB:
    case FIELD_TYPE_BLOB:       return "Blob:";     break;
    default:
      writeLog(p->logwindow,"Unknown fieldtype %d detected !\n",
	       p->fields[index].type);
      return "Unknown"; break;
   }
}

void setFieldColumnWidths(Params *p, int numcolumns, gchar **titles)
{
   int x;
   int fontwidth = 8; /* fixed fontsize for the time being */
   int colwidth;
   
   /* set column sizes according to the fieldlength : */
   for(x=0; x < numcolumns; x++)
   {
      colwidth = (strlen(titles[x]) + 5) * fontwidth;
      ldebug(3,"Column width for column %d : %d\n",x,colwidth);
      gtk_clist_set_column_width(GTK_CLIST(p->browserwindow), x, colwidth);
   }
}

void showFields(Params *p)
{
   int   x,numrows;
   int   numcolumns = 6;
   gchar length[10];
   gchar *row[6];
   gchar *titles[6] =
   { "Fieldname", "Tablename", "Length", "Type", "Unique", "Not Null" };
   
   ldebug(2,"showFields entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show fields: select a table first !\n");
      return;
   }
   
   numrows = p->numflds; /* number of browserlines */
   
   destroyBrowser(p);
   createBrowser(p,titles,field_selected,numcolumns);
   setFieldColumnWidths(p,numcolumns,titles);
   
   gtk_clist_freeze(GTK_CLIST(p->browserwindow));
   
   for(x=0; x < numrows; x++)
   {
      snprintf(length,sizeof(length) - 1,"%8d",p->fields[x].length);
      
      row[0] = p->fields[x].name;
      row[1] = p->table;
      row[2] = length;
      row[3] = getFieldTypeString(p, x);
      row[4] = IS_UNIQUE(p->fields[x].flags) ? "Y" : "N";
      row[5] = IS_NOT_NULL(p->fields[x].flags) ? "Y" : "N";
      ldebug(2,"field %d: type %d\n",x,p->fields[x].type);

      gtk_clist_append(GTK_CLIST(p->browserwindow),row);
      /* change the sortkey color : */
      if(x == p->sortkey)
	gtk_clist_set_row_style(GTK_CLIST(p->browserwindow), x, p->sortstyle);
      
      /* TODO : set color of sort field (primary key) */ 
   }
   
   gtk_clist_thaw(GTK_CLIST(p->browserwindow));
}

