#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

/* If we come here, then the user has selected a row in the list. */
void record_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
   Params *p = (Params *)data;

   ldebug(2,"record_selected entered\n");

   p->selectedrow = row; /* single click : just preserve selection */
   ldebug(3,"You selected row %d.\n ",row);
   
   /* This only catches a doubleclick !! : */
   if (event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"record_selected: doubleclick detected\n");
      
      if(p->table == NULL)
      {
	 showMessage("Currently no table selected !\n");
	 return;
      }
      
      ldebug(3,"You doubleclicked row %d. More specifically you clicked in "
	     "column %d\n\n", row, column);
      
      createModifyWindow(p);
   }
   else
      ldebug(3,"You selected row %d\n",row);
}

void destroyBrowser(Params *p)
{
   ldebug(2,"destroyBrowser entered\n");
   if(! GTK_IS_WIDGET(p->browserwindow))
   {
      p->browserwindow = NULL;
      return;
   }
   
   gtk_widget_hide(p->browserwindow);
   gtk_widget_destroy(p->browserwindow);

   p->browserinuse = FALSE;
   p->selectedrow = -1;
}

void createBrowser(Params *p, gchar **titles, void *callback, int size)
{
   GtkWidget *bw;
   
   ldebug(2,"createBrowser entered\n");
   
   bw = GTK_WIDGET(gtk_clist_new_with_titles(size, titles));
   
   gtk_signal_connect(GTK_OBJECT(bw), "select_row",
		      GTK_SIGNAL_FUNC(callback),
		      (gpointer) p);
   
   gtk_widget_ref (bw);
   gtk_object_set_data_full (GTK_OBJECT(p->mainwindow), "browserwindow", bw,
			     (GtkDestroyNotify) gtk_widget_unref);
   
   gtk_widget_show (bw);
  
   gtk_container_add(GTK_CONTAINER(p->browserframe), bw);
   
   p->browserwindow = bw;
   p->browserinuse = TRUE;
}

void setRecordColumnWidths(Params *p)
{
   int x;
   int fontwidth = 8; /* fixed fontsize for the time being */
   int colwidth;
   
   /* set column sizes according to the fieldlength : */
   for(x=0;x<p->numflds;x++)
   {
      colwidth = p->fields[x].length * fontwidth;
      ldebug(3,"Column width for column %d : %d\n",x,colwidth);
      gtk_clist_set_column_width(GTK_CLIST(p->browserwindow), x, colwidth);
   }
}

void showRecords(Params *p)
{
   int     x,numrows;
   SQL_ROW row;
   SQL_RES *res;
   gchar   **titles;
   char    query[MAXQUERY];
   
   ldebug(2,"showRecords entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show records:\n select a table first !\n");
      return;
   }
   
   snprintf(query,sizeof(query) - 1,SELECT ORDER,
	    p->table,p->fields[p->sortkey].name);
   
   if (executeQuery(p->logwindow, p->dbsock, query))
     return;
   
   res     = sql_store_result(p->dbsock);
   numrows = sql_num_rows(res);   /* get number of records in selection */

   titles = (gchar **)g_malloc(sizeof(gchar *) * p->numflds);
   for(x=0;x<p->numflds;x++)
     titles[x] = p->fields[x].name;

   destroyBrowser(p); /* destroy the possible previous instance */
   createBrowser(p,titles,record_selected,p->numflds);
   setRecordColumnWidths(p);
   g_free(titles);
   
   gtk_clist_freeze(GTK_CLIST(p->browserwindow));
   
   for(x=0;x < numrows;x++)
   {
      if((row = sql_fetch_row(res)) == NULL)
      {
	 showSqlError(p->logwindow,p->dbsock,"Error reading record %d !",x);
	 return;
      }
      
      /* write a row to the browser : */
      gtk_clist_append(GTK_CLIST(p->browserwindow),row);
   }
   
   gtk_clist_thaw(GTK_CLIST(p->browserwindow));
}

