#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

char *addStringQuotes(int type)
{
   if(
      type == FIELD_TYPE_DATE       || type == FIELD_TYPE_TIME   ||
      type == FIELD_TYPE_DATETIME   || type == FIELD_TYPE_STRING ||
      type == FIELD_TYPE_VAR_STRING
      )
     return "'";
   else
     return "";
}

void updateRecordAction(GtkButton *button, gpointer rec)
{
   Record *r = (Record *)rec;
   char *query;
   int y;
   int size = 20; /* good start */
   
   ldebug(2,"updateRecordAction entered\n");
   
   /* get approximate querylength: account for quotes,spaces and equalsigns */
   for(y=0;y<r->numflds;y++)
     size += 2 * (strlen(r->fields[y].name) + r->fields[y].length) + 10;
   query =(char *)g_malloc(size * sizeof(char));
   
   snprintf(query, size - 1, UPDATE , r->table);
   
   /* new values : */
   for(y=0;y<r->numflds;y++)
   {
      /* text entry can currently be larger than fieldlength: we 
       need strncat to truncate and thus avoid buffer overflows */
   
      strcat (query + strlen(query), r->fields[y].name);
      strcat (query + strlen(query),"=");
      strcat (query + strlen(query),addStringQuotes(r->fields[y].type));
      strncat(query + strlen(query),gtk_entry_get_text(r->entries[y]),
	      r->fields[y].length);
      strcat (query + strlen(query),addStringQuotes(r->fields[y].type));
      
      if(y< r->numflds - 1)
	strcat(query,",");
   }
   strcat(query, WHERE);
   
   /* old values */
   for(y=0;y<r->numflds;y++)
   {
      snprintf(query + strlen(query), size - strlen(query) - 10,
	       EQUAL ,
	       r->fields[y].name,
	       addStringQuotes(r->fields[y].type),
	       r->contents[y],
	       addStringQuotes(r->fields[y].type));
      
      if(y< r->numflds - 1)
	strcat(query,AND);
   }
   
   ldebug(3,"query space: %d, actual querylength : %d\n",size, strlen(query));
   
   if( executeQuery(r->logwindow, r->dbsock, query) == 0)
   {  
      writeLog(r->logwindow,"Record update succesful\n");
      refreshBrowser();
   }
   else
     writeLog(r->logwindow,"Record update failed\n");
   
   defaultAction(button, NULL); /* close dialog window */
   destroyRecordWindow(r); /* close record window */
   g_free(query);
}

void deleteRecordAction(GtkButton *button, gpointer rec)
{
   Record *r = (Record *)rec;
   char *query;
   int y;
   int size = 20; /* good start */
   
   ldebug(2,"deleteRecordAction entered\n");
   
   /* get approximate querylength: account for quotes,spaces and equalsigns */
   for(y=0;y<r->numflds;y++)
     size += strlen(r->fields[y].name) + r->fields[y].length + 10;
   query =(char *)g_malloc(size * sizeof(char));
   
   snprintf(query, size - 1, DELETE WHERE , r->table);
   
   for(y=0;y<r->numflds;y++)
   {
      snprintf(query + strlen(query), size - strlen(query) - 10,
	       EQUAL ,
	       r->fields[y].name,
	       addStringQuotes(r->fields[y].type),
	       r->contents[y],
	       addStringQuotes(r->fields[y].type));

      if(y< r->numflds - 1)
	strcat(query,AND);
   }
   
   ldebug(3,"query space: %d, actual querylength : %d\n",size, strlen(query));

   if( executeQuery(r->logwindow, r->dbsock, query) == 0)
   {
      writeLog(r->logwindow,"Record deletion succesful\n");
      refreshBrowser();
    }
   else
     writeLog(r->logwindow,"Record deletion failed\n");
   
   defaultAction(button, NULL); /* close dialog window */
   destroyRecordWindow(r); /* close record window */
   g_free(query);
}

void insertRecordAction(GtkButton *button, gpointer rec)
{
   Record *r = (Record *)rec;
   char *query;
   int y;
   int size=20; /* good start */
   
   ldebug(2,"insertRecordAction entered\n");
   
   /* get approximate querylength: account for quotes,spaces and equalsigns */
   for(y=0;y<r->numflds;y++)
     size += strlen(r->fields[y].name) + r->fields[y].length + 10;
   query =(char *)g_malloc(size * sizeof(char));
   
   snprintf(query,size -1,INSERT,r->table);
   
   for(y=0;y<r->numflds;y++)
   {
      /* text entry can currently be larger than fieldlength: we 
       need strncat to truncate and thus avoid buffer overflows */
   
      strcat (query + strlen(query),addStringQuotes(r->fields[y].type));
      strncat(query + strlen(query),gtk_entry_get_text(r->entries[y]),
	      r->fields[y].length);
      strcat (query + strlen(query),addStringQuotes(r->fields[y].type));
      
      if(y< r->numflds - 1)
	strcat(query,",");
   }
   strcat(query,")");
   
   ldebug(3,"query space: %d, actual querylength : %d\n",size, strlen(query));

   if( executeQuery(r->logwindow, r->dbsock, query) == 0)
   {
      writeLog(r->logwindow,"Record insertion succesful\n");
      refreshBrowser();
   }else
     writeLog(r->logwindow,"Record insertion failed\n");
   
   defaultAction(button, NULL); /* close dialog window */
//   destroyRecordWindow(r); /* close record window */
   g_free(query);
}

