#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"

#include "gequel.h"

/* Some NON-GLADE generated callbacks : */
void on_database_activate(GtkMenuItem *item, gpointer  user_data)
{
   Params *p = (Params *)user_data;
   GtkBin *menu = GTK_BIN(p->dbaselist);
   GtkLabel *label = GTK_LABEL(menu->child);
   char *text;
   
   gtk_label_get(label,&text); /* obtain label text of the selected item */
   ldebug(2,"on_database_activate called\n");
   setNewDatabase((Params *)user_data, text);
}

void on_table_activate(GtkMenuItem *item, gpointer  user_data)
{
   Params *p = (Params *)user_data;
   GtkBin *menu = GTK_BIN(p->tablelist);
   GtkLabel *label = GTK_LABEL(menu->child);
   char *text;
   
   gtk_label_get(label,&text); /* obtain label text of the selected item */
   ldebug(2,"on_table_activate called\n");
   setNewTable((Params *)user_data, text);
}

void releaseFieldInfo(Params *p)
{
   int x;
   
   ldebug(2,"releaseFieldInfo called\n");
   if(p->fields == NULL)
     return;
   
   for(x=0;x<p->numflds;x++)
     g_free(p->fields[x].name);
   g_free(p->fields);
   p->fields = NULL;
   
   setGUICounters(p,0,0);
}

void getFieldInfo(Params *p)
{
   int   x;
   int tlen;
   SQL_RES *res;
   
   ldebug(2,"getFieldInfo called\n");
   releaseFieldInfo(p);
   
   res = sql_list_fields(p->dbsock,p->table, 0);
   if (res == NULL)
   {
      showSqlError(p->logwindow,p->dbsock,
		   "Cannot fetch fieldnames for %s",p->table);
      p->table = NULL;
      return;
   }
   
   p->numflds = sql_num_fields(res); /* get number of fields */
   p->fields = (SQL_FIELD *)g_malloc(sizeof(SQL_FIELD) * p->numflds);
   if(p->fields == NULL)
   {
      showMessage("Memory error in create Params (fieldbuffer) for %s !",
		  p->table);
      exit(1);
   }
   
   for(x=0; x<p->numflds;x++)
   {
      SQL_FIELD *field;
      
      field = sql_fetch_field(res);
      
      /* copy the complete field struct to an array element : */
      memcpy(&p->fields[x],field,sizeof(SQL_FIELD));

      /* now save (copy) the pointer elements (name and table) */
      /* as they'll be lost after release of the selection */
      tlen = strlen(p->fields[x].name) + 1;
      p->fields[x].name = (char *)g_malloc(tlen);
      if(p->fields[x].name == NULL)
      {
	 showMessage("Memory error in create Params (names) for %s !",
		     p->table);
	 exit(1);
      }
      /* Copy pointer "name" as well. Note that we do not copy other strings 
       like "table" and "def" here, so fieldinfo is incomplete ! */
      
      strncpy(p->fields[x].name, field->name, tlen);
   }
   
   getSortKey(p);
   setGUICounters(p, p->numflds, p->numrecs);
   sql_free_result(res);
}

/* Remove the table list, clear the previous table and set counters to 0 */
void clearTableList(Params *p)
{
   GtkWidget *newmenu;
   GtkWidget *menulabel;
   
   ldebug(2,"clearTableList called\n");
   p->table = NULL;
   
   releaseFieldInfo(p); /* Free previous field info to avoid memory leaks */
   
   gtk_option_menu_remove_menu (p->tablelist);
   
   newmenu = gtk_menu_new();
   menulabel = gtk_menu_item_new_with_label("* None *");
   gtk_widget_show (menulabel);

   gtk_menu_append(GTK_MENU(newmenu), menulabel);   
   gtk_option_menu_set_menu(p->tablelist,newmenu);
}

/* Remove database list and previous database and then call clearTableList
 to remove the table list and to clear the previous table */
void clearDBaseList(Params *p)
{
   GtkWidget *newmenu;
   GtkWidget *menulabel;
   
   ldebug(2,"clearDBaseList called\n");
   clearTableList(p);
   p->database = NULL;

   gtk_option_menu_remove_menu (p->dbaselist);
   
   newmenu = gtk_menu_new();
   menulabel = gtk_menu_item_new_with_label("* None *");
   gtk_widget_show (menulabel);

   gtk_menu_append(GTK_MENU(newmenu), menulabel);   
   gtk_option_menu_set_menu(p->dbaselist,newmenu);
   
}

/* Clear previous table and refresh the table list to contain the one obtained
 from the currently selected database, optionally preselected a table */
void setTableList(Params *p, char *preselect)
{
   int       counter, preset = 0;
   SQL_RES   *res;
   SQL_ROW    row;
   GtkWidget *newmenu;
   GtkWidget *menulabel;
   
   ldebug(2,"setTableList called\n");
   p->table = NULL;
   releaseFieldInfo(p); /* Free previous field info to avoid memory leaks */
   
   if (p->dbsock == NULL)
   {
      showMessage("Cannot fetch tables: no connection");
      return;
   }
   
   res = sql_list_tables(p->dbsock,0);
   if(res == NULL)
   {
      showSqlError(p->logwindow, p->dbsock,
		   "Cannot fetch tables for database %s",p->database);
      return;
   }
   
   gtk_option_menu_remove_menu(p->tablelist);
   
   newmenu = gtk_menu_new();
   menulabel = gtk_menu_item_new_with_label("* Select *");
   gtk_widget_show (menulabel);
   gtk_menu_append(GTK_MENU(newmenu), menulabel);

   counter = 1;
   while((row = sql_fetch_row(res)) != NULL)
   {
      menulabel = gtk_menu_item_new_with_label(row[0]);
      gtk_widget_show (menulabel);
      gtk_menu_append(GTK_MENU(newmenu), menulabel);
      gtk_signal_connect
	(GTK_OBJECT (menulabel), "activate",
	 GTK_SIGNAL_FUNC (on_table_activate),
	 (gpointer) p);
      
      /* set current table to preselect in optionmenu */
      if(preselect != NULL)
      {
	 if(strcmp(row[0],preselect) == 0)
	   preset = counter;
	 counter++;
      }
      
      ldebug(3,"table: %s\n",row[0]);
   }
   gtk_option_menu_set_menu(p->tablelist,newmenu);
   gtk_option_menu_set_history (p->tablelist, preset);
   
   sql_free_result(res);
}

/* Call clearTableList to clear the table list and previous table, clear the
 curent database and refresh the database list to contain the one obtained
 from the current server */
void setDBaseList(Params *p, char *preselect)
{
   int       counter, preset = 0;
   SQL_RES    *res;
   SQL_ROW    row;
   GtkWidget *newmenu;
   GtkWidget *menulabel;
   
   ldebug(2,"setDBaseList called\n");
   p->database = NULL;
   clearTableList(p);
   
   if (p->dbsock == NULL)
   {
      showMessage("Cannot fetch databases: no connection");
      return;
   }
   
   res = sql_list_dbs(p->dbsock,0);
   if(res == NULL)
   {
      showSqlError(p->logwindow,p->dbsock,"Cannot fetch databases from server"
		   " %s",p->server == NULL ? "<UNIX socket>" : p->server);
      return;
   }
   
   gtk_option_menu_remove_menu (p->dbaselist);
   
   newmenu = gtk_menu_new();
   menulabel = gtk_menu_item_new_with_label("* Select *");
   gtk_widget_show (menulabel);
   gtk_menu_append(GTK_MENU(newmenu), menulabel);
   
   counter = 1;
   while((row = sql_fetch_row(res)) != NULL)
   {
      menulabel = gtk_menu_item_new_with_label(row[0]);
      gtk_widget_show (menulabel);
      gtk_menu_append(GTK_MENU(newmenu), menulabel);
      gtk_signal_connect
	(GTK_OBJECT (menulabel), "activate",
	 GTK_SIGNAL_FUNC (on_database_activate),
	 (gpointer) p);
      
      if(preselect != NULL)
      {
	 if(strcmp(row[0],preselect) == 0)
	    preset = counter;
	 counter++;
      }
      
      ldebug(3,"database: %s\n",row[0]);
   }
   gtk_option_menu_set_menu(p->dbaselist,newmenu);
   gtk_option_menu_set_history (p->dbaselist, preset);

   sql_free_result(res);
}

/* Select a new table, read table info (fields) and set the GUI counters */
void setNewTable(Params *p, char *table)
{
   ldebug(2,"setNewTable called\n");
   if(p->browserinuse == TRUE) /* remove browser contents if in use */
     destroyBrowser(p);

   p->table = table;  /* strncpy ? */
   
   if(table == NULL)
      return;
   
   getNumRecords(p);
   getFieldInfo(p);
}

/* Use a new database, clear the previous table, fill the tablelist
 * with tables from this database and set the table counters to 0 */
void setNewDatabase(Params *p, char *database)
{
   ldebug(2,"setNewDatabase called\n");
   if(p->browserinuse == TRUE) /* remove browser contents if in use */
     destroyBrowser(p);

   p->database = database; /* strncpy ? */

   if(database == NULL)
     return;
   
   if(sql_select_db(p->dbsock,database) < 0)
   {
      showSqlError(p->logwindow,p->dbsock,
		   "Error opening database %s",database);
      clearTableList(p);
      return;
   }
   p->numrecs = 0;
   
   setTableList(p, NULL);
}

/* Connect to a database server, set the server and list of available
 databases, clear tablelist and clear previous table and database.
 A NULL server will connect to the localhost. */
void setNewServer(Params *p, char *server)
{
   ldebug(2,"setNewServer called\n");
   p->server = server;
   if(p->server != NULL)
   {
      if (strlen(p->server) == 0)
	p->server = NULL;  /* provide for empty but not NULL string */
      else
	setStringValue(p->mainwindow, "server", p->server);
   }

   p->dbsock=mysql_init(NULL);
   if (p->dbsock == NULL || mysql_real_connect
       (p->dbsock, p->server, p->user, p->pwd, NULL, 0, NULL, 0) == NULL)
   {
      showSqlError(p->logwindow, p->dbsock, "Error connecting to server");
      clearDBaseList(p);
   }
   else
     setDBaseList(p, NULL);
}

